package datavisualizer;

import javax.swing.JInternalFrame;
import javax.swing.event.*;

/**
 * Ensures consistent behavior across graph windows.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class ModelInternalFrame extends JInternalFrame implements javax.swing.event.InternalFrameListener {

  /**This window's legend.  Focusing this window causes its legend to come to
   * the front.*/
  private JInternalFrame m_jLegend;

  /**
   * Constructor.  Creates a window that is iconifiable, resizable, and
   * closeable.
   * @param sTitle String Title string.
   * @param jLegend JInternalFrame Legend to bring to focus.
   */
  public ModelInternalFrame(String sTitle, JInternalFrame jLegend) {
    super(sTitle, true, true, true, true);
    m_jLegend = jLegend;
    this.addInternalFrameListener(this);
  }

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameClosed(InternalFrameEvent e){}

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameClosing(InternalFrameEvent e) {}

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameDeactivated(InternalFrameEvent e) {}

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameDeiconified(InternalFrameEvent e) {}

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameIconified(InternalFrameEvent e) {}

  /**
   * Not overridden.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameOpened(InternalFrameEvent e) {}

  /**
   * Brings this window's legend to front when this window is activated.
   * @param e InternalFrameEvent Ignored.
   */
  public void internalFrameActivated(InternalFrameEvent e) {
    m_jLegend.toFront();
    this.toFront();
  }
}
