package datavisualizer;

import java.io.Serializable;

/**
 * Represents one (x, y, z) data item for an xyz-series.  Originally from
 * JFreeChart - I modified it.
 *  Original author - David Gilbert
 * <p>Title: Data Visualizer</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class XYZDataItem
    implements Serializable {
  /** The x-value. */
  private Number x;

  /** The y-value. */
  private Number y;

  /** The z-value. */
  private Number z;

  /**
   * Constructs a new data pair.
   *
   * @param x  the x-value.
   * @param y  the y-value.
   * @param z  the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZDataItem(Number x, Number y, Number z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }

  /**
   * Constructs a new data pair.
   *
   * @param x  the x-value.
   * @param y  the y-value.
   * @param z  the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public XYZDataItem(double x, double y, double z) {
    this(new Double(x), new Double(y), new Double(z));
  }

  /**
   * Returns the x-value.
   *
   * @return the x-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getX() {
    return this.x;
  }

  /**
   * Returns the y-value.
   *
   * @return the y-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getY() {
    return this.y;
  }

  /**
   * Returns the z-value.
   *
   * @return the z-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Number getZ() {
    return this.z;
  }

  /**
   * Sets the y-value for this data pair.
   *
   * Note that there is no corresponding method to change the x-value.
   *
   * @param y  the new y-value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void setY(Number y) {
    this.y = y;
  }

  /**
   * Sets the z-value for this data pair.
   *
   * Note that there is no corresponding method to change the x-value.
   *
   * @param z  the new z-value.
   */
  public void setZ(Number z) {
    this.z = z;
  }
}
