package javawrapper;

/**
 * Tests the DisperseBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

 */
public class TestDisperseBehaviors
    extends ModelTestCase {


  /**
   * Tests to make sure parameters are correctly displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      DisperseBehaviors oDisperse = null;
      TreePopulation oPop = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no disperse behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      assertEquals(null, oDisperse.FormatDataForDisplay(oPop));

      //Case:  only non-spatial disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("non spatial disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  no gap disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("non-gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "STR for Stumps",
          "Beta for Stumps",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Weibull Canopy Annual STR",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  only gap disperse is enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oDisperse.GetBehaviorByKey("gap disperse");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, oPop));
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Weibull Gap Annual STR",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Lognormal Gap Annual STR",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoDisperseXMLFile());
      oDisperse = oManager.GetDisperseBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oDisperse.mp_oChildBehaviors.length; i++) {
        oDisperse.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oDisperse.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(
            0, 3, oPop));
      }
      p_sExpected = new String[] {
          "Minimum DBH for Reproduction, in cm",
          "Maximum Parent Trees Allowed in Gap Cell",
          "Weibull Gap Annual STR",
          "Weibull Gap Beta",
          "Weibull Gap Theta",
          "Weibull Gap Dispersal",
          "Lognormal Gap Annual STR",
          "Lognormal Gap Beta",
          "Lognormal Gap Xb",
          "Lognormal Gap X0",
          "Lognormal Canopy Annual STR",
          "Lognormal Canopy Beta",
          "Lognormal Canopy Xb",
          "Lognormal Canopy X0",
          "Canopy Function Used",
          "Gap Function Used",
          "STR for Stumps",
          "Beta for Stumps",
          "Weibull Canopy Theta",
          "Weibull Canopy Dispersal",
          "Weibull Canopy Beta",
          "Weibull Canopy Annual STR",
          "Slope Mean Non-Spatial Seed Rain, seeds/m2/ha of BA/yr",
          "Intercept of Mean Non-Spatial Seed Rain, seeds/m2/yr",
          "Seed Dist. Std. Deviation (Normal or Lognormal)",
          "Seed Dist. Clumping Parameter (Neg. Binomial)",
          "Seed Distribution",
          "Azimuth Direction of Max Dispersal Distance, in rad",
          "Amplitude of Anisotropic Effect",
          "Maximum Search Distance for Neighbor Parents, in m"
      };
      TestTable(oDisperse.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for disperse.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.sMessage);
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a file with no disperse settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 18, 2006: Created (LEM)
   */
  protected String WriteNoDisperseXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}
