package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;
import java.util.Vector;
import java.io.File;

/**
 * Tests ParseReader class
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestParseReader
    extends TestCase {

  /**
   * Tests the XMLTrim method.
   */
  public void testXMLTrim() {
    //Case - null string
    assertEquals(null, SaxParseTools.XMLTrim(null));

    //Case - string of no length
    assertEquals("", SaxParseTools.XMLTrim(""));

    //Case - string with no trim characters
    assertEquals("test", SaxParseTools.XMLTrim("test"));

    //Case - string that is all spaces
//    assertEquals("", SaxParseTools.XMLTrim("    "));

    //Case - string that is all end-of-lines
    assertEquals("", SaxParseTools.XMLTrim("\n\n\n"));

    //Case - string that is all returns
    assertEquals("", SaxParseTools.XMLTrim("\r\r\r"));

    //Case - string that is all tabs
    assertEquals("", SaxParseTools.XMLTrim("\t\t\t"));

    //Case - string that begins with spaces
//    assertEquals("test", SaxParseTools.XMLTrim("    test"));

    //Case - string that begins with end-of-lines
    assertEquals("test", SaxParseTools.XMLTrim("\n\ntest"));

    //Case - string that begins with returns
    assertEquals("test", SaxParseTools.XMLTrim("\rtest"));

    //Case - string that begins with tabs
    assertEquals("test", SaxParseTools.XMLTrim("\t\t\t\t\ttest"));

    //Case - string that ends with spaces
//    assertEquals("test", SaxParseTools.XMLTrim("test "));

    //Case - string that ends with end-of-lines
    assertEquals("test", SaxParseTools.XMLTrim("test\n\n\n"));

    //Case - string that ends with returns
    assertEquals("test", SaxParseTools.XMLTrim("test\r\r"));

    //Case - string that ends with tabs
    assertEquals("test", SaxParseTools.XMLTrim("test\t\t\t"));

    //Case - string that beings and ends with mix
    assertEquals(" test ", SaxParseTools.XMLTrim("\t\r\n test \n\r\t\t"));

    System.out.println("XMLTrim test successful.");

  }

  /**
   * Tests basic parsing and value assignment of an XML file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testParsing() {
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      String sFileName = WriteXMLFile1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      //Now check the data

      //Single values - random seed and number of timesteps
      Plot oPlot = oManager.GetPlot();
      assertEquals(100, oPlot.m_iNumTimesteps.GetValue());
      assertEquals(3, oPlot.m_iRandomSeed.GetValue());

      GrowthBehaviors oGrowth = oManager.GetGrowthBehaviors();

      //Test the vector with all values for all species
      Vector p_oVector = oGrowth.mp_fAdultConstantAreaInc.GetValue();
      Float fVal = (Float) p_oVector.elementAt(0);
      assertEquals( (float) 1.18, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(1);
      assertEquals( (float) 1.53, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(2);
      assertEquals( (float) 0.92, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(3);
      assertEquals( (float) 0.98, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(4);
      assertEquals( (float) 1.6, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(5);
      assertEquals( (float) 0.93, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(6);
      assertEquals( (float) 1.14, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(7);
      assertEquals( (float) 2.47, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(8);
      assertEquals( (float) 0.72, fVal.floatValue(), 0.0001);

      //Test the vector with no values for any species
      p_oVector = oGrowth.mp_fAsymptoticDiameterGrowth.GetValue();
      assertEquals(0, p_oVector.size());

      //Test the vector with values for some species
      p_oVector = oGrowth.mp_fSlopeOfGrowthResponse.GetValue();
      fVal = (Float) p_oVector.elementAt(0);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(1);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(2);
      assertEquals( (float) 0.01, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(3);
      assertEquals( (float) 0.015, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(4);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(5);
      assertEquals( (float) 0.0090, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(6);
      assertEquals( (float) 0.034, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(7);
      assertEquals( (float) 0.035, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(8);
      assertEquals(null, fVal);

      //Test the vector with values for all species, not in order
      p_oVector = oGrowth.mp_fLengthOfLastSuppressionFactor.GetValue();
      fVal = (Float) p_oVector.elementAt(0);
      assertEquals( (float) 0.5, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(1);
      assertEquals( (float) 0.3, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(2);
      assertEquals( (float) 0.1, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(3);
      assertEquals( (float) 0.4, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(4);
      assertEquals( (float) 0.2, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(5);
      assertEquals( (float) 0.0050, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(6);
      assertEquals( (float) 0.024, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(7);
      assertEquals( (float) 0.011, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(8);
      assertEquals( (float) 0.016, fVal.floatValue(), 0.0001);

      //Test the vector with values for some species, not in order
      p_oVector = oGrowth.mp_fLengthOfCurrentReleaseFactor.GetValue();
      fVal = (Float) p_oVector.elementAt(0);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(1);
      assertEquals( (float) 0.1, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(2);
      assertEquals( (float) 0.016, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(3);
      assertEquals( (float) 0.4, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(4);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(5);
      assertEquals( (float) 0.02, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(6);
      assertEquals( (float) 0.3, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(7);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(8);
      assertEquals(null, fVal);

      //Test the vector with value for one species
      p_oVector = oGrowth.mp_fNCIMaxCrowdingRadius.GetValue();
      fVal = (Float) p_oVector.elementAt(0);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(1);
      assertEquals( (float) 12.87, fVal.floatValue(), 0.0001);
      fVal = (Float) p_oVector.elementAt(2);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(3);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(4);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(5);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(6);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(7);
      assertEquals(null, fVal);
      fVal = (Float) p_oVector.elementAt(8);
      assertEquals(null, fVal);

      new File(sFileName).delete();
    }
    catch (ModelException oErr) {
      fail("Parse test failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * This writes an XML file to test value setting.
   * @return Name of file.
   * @throws ModelException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected String WriteXMLFile1() throws ModelException {
    try {
      String sFileName = "\\loratestxml1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write(
          "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
      oOut.write("<paramFile fileCode=\"06010101\">");
      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"Western_Hemlock\"/>");
      oOut.write("<tr_species speciesName=\"Western_Redcedar\"/>");
      oOut.write("<tr_species speciesName=\"Amabilis_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Subalpine_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Hybrid_Spruce\"/>");
      oOut.write("<tr_species speciesName=\"Lodgepole_Pine\"/>");
      oOut.write("<tr_species speciesName=\"Trembling_Aspen\"/>");
      oOut.write("<tr_species speciesName=\"Black_Cottonwood\"/>");
      oOut.write("<tr_species speciesName=\"Paper_Birch\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("</trees>");

      //Include an empty behavior list - this triggers setup
      oOut.write("<behaviorList>");
      oOut.write("</behaviorList>");

      //Concentrate on growth
      oOut.write("<growth>");

      //The softball:  Vector value for every species, in order
      oOut.write("<gr_adultConstAreaInc>");
      oOut.write("<gr_acaiVal species=\"Western_Hemlock\">1.18</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Western_Redcedar\">1.53</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Amabilis_Fir\">0.92</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Subalpine_Fir\">0.98</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Hybrid_Spruce\">1.6</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Lodgepole_Pine\">0.93</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Trembling_Aspen\">1.14</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Black_Cottonwood\">2.47</gr_acaiVal>");
      oOut.write("<gr_acaiVal species=\"Paper_Birch\">0.72</gr_acaiVal>");
      oOut.write("</gr_adultConstAreaInc>");

      //No values for any species
      oOut.write("<gr_asympDiameterGrowth>");
      oOut.write("</gr_asympDiameterGrowth>");

      //Values for some species
      oOut.write("<gr_slopeGrowthResponse>");
      oOut.write("<gr_sgrVal species=\"Amabilis_Fir\">0.01</gr_sgrVal>");
      oOut.write("<gr_sgrVal species=\"Subalpine_Fir\">0.015</gr_sgrVal>");
      oOut.write("<gr_sgrVal species=\"Lodgepole_Pine\">0.0090</gr_sgrVal>");
      oOut.write("<gr_sgrVal species=\"Trembling_Aspen\">0.034</gr_sgrVal>");
      oOut.write("<gr_sgrVal species=\"Black_Cottonwood\">0.035</gr_sgrVal>");
      oOut.write("</gr_slopeGrowthResponse>");

      //Sprinkle in a single value
      oOut.write("<timesteps>100</timesteps>");

      //Values for all species, not in order
      oOut.write("<gr_lengthLastSuppFactor>");
      oOut.write("<gr_llsfVal species=\"Amabilis_Fir\">0.1</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Hybrid_Spruce\">0.2</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Western_Redcedar\">0.3</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Trembling_Aspen\">0.024</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Subalpine_Fir\">0.4</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Paper_Birch\">0.016</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Lodgepole_Pine\">0.0050</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Black_Cottonwood\">0.011</gr_llsfVal>");
      oOut.write("<gr_llsfVal species=\"Western_Hemlock\">0.5</gr_llsfVal>");
      oOut.write("</gr_lengthLastSuppFactor>");

      //Values for some species, not in order
      oOut.write("<gr_lengthCurrReleaseFactor>");
      oOut.write("<gr_lcrfVal species=\"Trembling_Aspen\">0.3</gr_lcrfVal>");
      oOut.write("<gr_lcrfVal species=\"Amabilis_Fir\">0.016</gr_lcrfVal>");
      oOut.write("<gr_lcrfVal species=\"Western_Redcedar\">0.1</gr_lcrfVal>");
      oOut.write("<gr_lcrfVal species=\"Lodgepole_Pine\">0.02</gr_lcrfVal>");
      oOut.write("<gr_lcrfVal species=\"Subalpine_Fir\">0.4</gr_lcrfVal>");
      oOut.write("</gr_lengthCurrReleaseFactor>");

      //Sprinkle in a single value
      oOut.write("<randomSeed>3</randomSeed>");

      //Values for one species
      oOut.write("<gr_nciMaxCrowdingRadius>");
      oOut.write("<gr_nmcrVal species=\"Western_Redcedar\">12.87</gr_nmcrVal>");
      oOut.write("</gr_nciMaxCrowdingRadius>");

      oOut.write("</growth>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }

}
